@@ -2,35 +2,124 @@ import ast
 import re
 from ast import expr
 
-from sympy import Add, Mul, Pow, Eq, Function
+from sympy import Add, Mul, Pow, Eq
 
 from expr_codegen.expr import register_symbols, dict_to_exprs
 
 
-class SympyTransformer(ast.NodeTransformer):
-    """将ast转换成Sympy要求的格式"""
+class SyntaxTransformer(ast.NodeTransformer):
+    """修改语法。注意：一定要修改语法后才能改名"""
 
-    # 旧记录
-    funcs_old = set()
-    args_old = set()
-    targets_old = set()
-    # 旧记录
-    funcs_new = set()
-    args_new = set()
-    targets_new = set()
-
-    # 映射
-    funcs_map = {}
-    # 由于None等常量无法在sympy中正确处理，只能改成Symbol变量
-    # !!!一定要在drop_symbols时排除
-    args_map = {'True': "_TRUE_", 'False': "_FALSE_", 'None': "_NONE_"}
-    targets_map = {}  # 只对非下划线开头的生效
     # ^ 是异或还是乘方呢？
     convert_xor: bool = True
 
-    def config_map(self, funcs_map, args_map, targets_map):
+    def visit_Compare(self, node):
+        assert len(node.comparators) == 1, f"不支持连续等号，请手工添加括号, {ast.unparse(node)}"
+
+        self.generic_visit(node)
+        return node
+
+    def visit_IfExp(self, node):
+        # 三元表达式。需要在外部提前替换成or True if else
+        # 只要body区域，出现了or True，就认为是特殊处理过的
+        if isinstance(node.body, ast.BoolOp) and isinstance(node.body.op, ast.Or):
+            if isinstance(node.body.values[-1], ast.Constant):
+                if node.body.values[-1].value:
+                    node.test, node.body = node.body.values[0], node.test
+
+        node = ast.Call(
+            func=ast.Name(id='if_else', ctx=ast.Load()),
+            args=[node.test, node.body, node.orelse],
+            keywords=[],
+        )
+
+        self.generic_visit(node)
+        return node
+
+    def visit_BinOp(self, node):
+        # TypeError: unsupported operand type(s) for *: 'StrictLessThan' and 'int'
+        if isinstance(node.op, (ast.Mult, ast.Add, ast.Div, ast.Sub)):
+            # (OPEN < CLOSE) * -1
+            if isinstance(node.left, ast.Compare):
+                node.left = ast.Call(
+                    func=ast.Name(id='int_', ctx=ast.Load()),
+                    args=[node.left],
+                    keywords=[],
+                )
+            # -1*(OPEN < CLOSE)
+            if isinstance(node.right, ast.Compare):
+                node.right = ast.Call(
+                    func=ast.Name(id='int_', ctx=ast.Load()),
+                    args=[node.right],
+                    keywords=[],
+                )
+            # 这种情况，已经包含
+            # (OPEN < CLOSE)*(OPEN < CLOSE)
+
+        if isinstance(node.op, ast.BitXor):
+            # ^ 运算符，转换为pow还是xor
+            if self.convert_xor:
+                node = ast.Call(
+                    func=ast.Name(id='Pow', ctx=ast.Load()),
+                    args=[node.left, node.right],
+                    keywords=[],
+                )
+            else:
+                node = ast.Call(
+                    func=ast.Name(id='Xor', ctx=ast.Load()),
+                    args=[node.left, node.right],
+                    keywords=[],
+                )
+
+        self.generic_visit(node)
+        return node
+
+    def visit_UnaryOp(self, node):
+        # ~ts_delay 报错，替换成Not(ts_delay)
+        if isinstance(node.op, ast.Invert):
+            node = ast.Call(
+                func=ast.Name(id='Not', ctx=ast.Load()),
+                args=[node.operand],
+                keywords=[],
+            )
+
+        self.generic_visit(node)
+        return node
+
+    def visit_Subscript(self, node):
+        if isinstance(node.slice, ast.Constant) and node.slice.value == 0:
+            node = node.value
+        elif isinstance(node.slice, ast.UnaryOp) and isinstance(node.slice.operand, ast.Constant) and node.slice.operand.value == 0:
+            node = node.value
+        else:
+            node = ast.Call(
+                func=ast.Name(id='ts_delay', ctx=ast.Load()),
+                args=[node.value, node.slice],
+                keywords=[],
+            )
+        self.generic_visit(node)
+        return node
+
+
+class RenameTransformer(ast.NodeTransformer):
+    """改名处理。改名前需要语法规范"""
+
+    def __init__(self, funcs_map, targets_map, args_map=None):
+
+        if args_map is None:
+            args_map = {'True': "_TRUE_", 'False': "_FALSE_", 'None': "_NONE_"}
+        self.funcs_old = set()
+        self.args_old = set()
+        self.targets_old = set()
+        self.funcs_new = set()
+        self.args_new = set()
+        self.targets_new = set()
+        # 映射
         self.funcs_map = funcs_map
+        # 由于None等常量无法在sympy中正确处理，只能改成Symbol变量
+        # !!!一定要在drop_symbols时排除
         self.args_map = args_map
+        # 只对非下划线开头的生效
         self.targets_map = targets_map
 
     def visit_Call(self, node):
@@ -115,6 +204,7 @@ class SympyTransformer(ast.NodeTransformer):
             self.args_old.add(node.left.id)
             node.left.id = self.args_map.get(node.left.id, node.left.id)
             self.args_new.add(node.left.id)
+
         for i, com in enumerate(node.comparators):
             if isinstance(com, ast.Name):
                 self.args_old.add(com.id)
@@ -128,44 +218,23 @@ class SympyTransformer(ast.NodeTransformer):
                     node.comparators[i] = ast.Name(new_com_value, ctx=ast.Load())
                     self.args_new.add(new_com_value)
 
-        assert len(node.comparators) == 1, f"不支持连续等号，请手工添加括号, {ast.unparse(node)}"
-
         self.generic_visit(node)
         return node
 
     def visit_IfExp(self, node):
-        # 三元表达式。需要在外部提前替换成if else
-        # OPEN>=CLOSE?1:0
-        # OPEN>CLOSE?A==B?3:DE>FG?5:6:0
-        node = ast.Call(
-            func=ast.Name(id='if_else', ctx=ast.Load()),
-            args=[node.body, node.test, node.orelse],
-            keywords=[],
-        )
+        if isinstance(node.body, ast.Name):
+            self.args_old.add(node.body.id)
+            node.body.id = self.args_map.get(node.body.id, node.body.id)
+            self.args_new.add(node.body.id)
+        if isinstance(node.orelse, ast.Name):
+            self.args_old.add(node.orelse.id)
+            node.orelse.id = self.args_map.get(node.orelse.id, node.orelse.id)
+            self.args_new.add(node.orelse.id)
 
         self.generic_visit(node)
         return node
 
     def visit_BinOp(self, node):
-        # TypeError: unsupported operand type(s) for *: 'StrictLessThan' and 'int'
-        if isinstance(node.op, (ast.Mult, ast.Add, ast.Div, ast.Sub)):
-            # (OPEN < CLOSE) * -1
-            if isinstance(node.left, ast.Compare):
-                node.left = ast.Call(
-                    func=ast.Name(id='int_', ctx=ast.Load()),
-                    args=[node.left],
-                    keywords=[],
-                )
-            # -1*(OPEN < CLOSE)
-            if isinstance(node.right, ast.Compare):
-                node.right = ast.Call(
-                    func=ast.Name(id='int_', ctx=ast.Load()),
-                    args=[node.right],
-                    keywords=[],
-                )
-            # 这种情况，已经包含
-            # (OPEN < CLOSE)*(OPEN < CLOSE)
-
         if isinstance(node.left, ast.Name):
             self.args_old.add(node.left.id)
             node.left.id = self.args_map.get(node.left.id, node.left.id)
@@ -189,21 +258,6 @@ class SympyTransformer(ast.NodeTransformer):
                 node.right = ast.Name(new_node_value, ctx=ast.Load())
                 self.args_new.add(new_node_value)
 
-        if isinstance(node.op, ast.BitXor):
-            # ^ 运算符，转换为pow还是xor
-            if self.convert_xor:
-                node = ast.Call(
-                    func=ast.Name(id='Pow', ctx=ast.Load()),
-                    args=[node.left, node.right],
-                    keywords=[],
-                )
-            else:
-                node = ast.Call(
-                    func=ast.Name(id='xor', ctx=ast.Load()),
-                    args=[node.left, node.right],
-                    keywords=[],
-                )
-
         self.generic_visit(node)
         return node
 
@@ -225,73 +279,27 @@ class SympyTransformer(ast.NodeTransformer):
         return node
 
     def visit_Subscript(self, node):
-        if node.slice.value == 0:
-            node = node.value
-        else:
-            node = ast.Call(
-                func=ast.Name(id='ts_delay', ctx=ast.Load()),
-                args=[node.value, node.slice],
-                keywords=[],
-            )
+        self.args_old.add(node.value.id)
+        node.value.id = self.args_map.get(node.value.id, node.value.id)
+        self.args_new.add(node.value.id)
+
         self.generic_visit(node)
         return node
 
 
-def sources_to_asts(*sources, convert_xor: bool):
-    """输入多份源代码"""
-    raw = []
-    assigns = {}
-    funcs_new, args_new, targets_new = set(), set(), set()
-    for arg in sources:
-        r, a, funcs_, args_, targets_ = _source_to_asts(arg, convert_xor)
-        raw.append(r)
-        assigns.update(a)
-        funcs_new.update(funcs_)
-        args_new.update(args_)
-        targets_new.update(targets_)
-    return '\n'.join(raw), assigns, funcs_new, args_new, targets_new
-
-
-def source_replace(source):
+def source_replace(source: str) -> str:
     # 三元表达式转换成 错误版if( )else，一定得在Transformer中修正
     num = 1
     while num > 0:
-        # A == B?D == E?1: 2:0 + 0
-        # 其实会导致?与:错配，但无所谓，只要多执行几次即可
-        source, num = re.subn(r'\?(.+?):(.+?)', r' if( \1 )else \2', source, flags=re.S)
+        # 利用or 的优先级最低，构造特殊的if else，只要出现，就认为位置要替换
+        # C?T:F --> C or True if( T )else F
+        source, num = re.subn(r'\?(.+?):(.+?)', r' or True if( \1 )else \2', source, flags=re.S)
         # break
     # 或、与
     source = source.replace('||', '|').replace('&&', '&')
     return source
 
 
-def _source_to_asts(source, convert_xor: bool):
-    """源代码"""
-    tree = ast.parse(source_replace(source))
-    t = SympyTransformer()
-    t.convert_xor = convert_xor
-    t.visit(tree)
-
-    raw = []
-    assigns = []
-
-    if isinstance(tree.body[0], ast.FunctionDef):
-        body = tree.body[0].body
-    else:
-        body = tree.body
-
-    for node in body:
-        # 特殊处理的节点
-        if isinstance(node, ast.Assign):
-            assigns.append(node)
-            continue
-        # TODO 是否要把其它语句也加入？是否有安全问题？
-        if isinstance(node, (ast.Import, ast.ImportFrom)):
-            raw.append(node)
-            continue
-    return raw_to_code(raw), assigns_to_dict(assigns), t.funcs_new, t.args_new, t.targets_new
-
-
 def assigns_to_dict(assigns):
     """赋值表达式转成字典"""
     return {ast.unparse(a.targets): ast.unparse(a.value) for a in assigns}
@@ -302,14 +310,51 @@ def raw_to_code(raw):
     return '\n'.join([ast.unparse(a) for a in raw])
 
 
+def sources_to_asts(*sources, convert_xor: bool):
+    """输入多份源代码"""
+
+    def _source_to_asts(source):
+        """源代码"""
+        tree = ast.parse(source_replace(source))
+
+        if isinstance(tree.body[0], ast.FunctionDef):
+            body = tree.body[0].body
+        else:
+            body = tree.body
+
+        return body
+
+    tree = ast.parse("")
+    for arg in sources:
+        tree.body.extend(_source_to_asts(arg))
+
+    t1 = SyntaxTransformer()
+    t1.convert_xor = convert_xor
+    t1.visit(tree)
+    t = RenameTransformer({}, {})
+    t.visit(tree)
+
+    raw = []
+    assigns = []
+
+    for node in tree.body:
+        # 特殊处理的节点
+        if isinstance(node, ast.Assign):
+            assigns.append(node)
+            continue
+        # TODO 是否要把其它语句也加入？是否有安全问题？
+        if isinstance(node, (ast.Import, ast.ImportFrom)):
+            raw.append(node)
+            continue
+    return raw_to_code(raw), assigns_to_dict(assigns), t.funcs_new, t.args_new, t.targets_new
+
+
 def _add_default_type(globals_):
     # 这种写法可以省去由用户导入Eq一类的工作
     globals_['Add'] = Add
     globals_['Mul'] = Mul
     globals_['Pow'] = Pow
     globals_['Eq'] = Eq
-    # 支持OPEN[1]
-    globals_['ts_delay'] = Function('ts_delay')
     return globals_
 
 
@@ -319,6 +364,9 @@ def sources_to_exprs(globals_, *sources, convert_xor: bool):
     globals_ = _add_default_type(globals_)
 
     raw, assigns, funcs_new, args_new, targets_new = sources_to_asts(*sources, convert_xor=convert_xor)
+    # 支持OPEN[1]转ts_delay(OPEN,1)
+    funcs_new.add('ts_delay')
+
     register_symbols(funcs_new, globals_, is_function=True)
     register_symbols(args_new, globals_, is_function=False)
     register_symbols(targets_new, globals_, is_function=False)
